@@ -1,3 +1,4 @@
+/* vi: set sw=4 ts=4: */
 /*
  *  Copyright (C) 2003 Glenn L. McGrath
  *  Copyright (C) 2003-2004 Erik Andersen
@@ -5,15 +6,7 @@
  * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
  */
 
-#include <fcntl.h>
-#include <limits.h>
-#include <stdio.h>
-#include <stdint.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-
-#include "busybox.h"
+#include "libbb.h"
 
 typedef enum { HASH_SHA1, HASH_MD5 } hash_algo_t;
 
@@ -23,17 +16,12 @@ typedef enum { HASH_SHA1, HASH_MD5 } hash_algo_t;
 
 /* This might be useful elsewhere */
 static unsigned char *hash_bin_to_hex(unsigned char *hash_value,
-									  unsigned char hash_length)
+				unsigned hash_length)
 {
-	int x, len, max;
-	unsigned char *hex_value;
-
-	max = (hash_length * 2) + 2;
-	hex_value = xmalloc(max);
-	for (x = len = 0; x < hash_length; x++) {
-		len += snprintf((char*)(hex_value + len), max - len, "%02x", hash_value[x]);
-	}
-	return (hex_value);
+	/* xzalloc zero-terminates */
+	char *hex_value = xzalloc((hash_length * 2) + 1);
+	bin2hex(hex_value, (char*)hash_value, hash_length);
+	return hex_value;
 }
 
 static uint8_t *hash_file(const char *filename, hash_algo_t hash_algo)
@@ -47,98 +35,98 @@ static uint8_t *hash_file(const char *filename, hash_algo_t hash_algo)
 	RESERVE_CONFIG_UBUFFER(in_buf, 4096);
 	void (*update)(const void*, size_t, void*);
 	void (*final)(void*, void*);
-	
-	if(strcmp(filename, "-") == 0) {
-		src_fd = STDIN_FILENO;
-	} else if(0 > (src_fd = open(filename, O_RDONLY))) {
-		bb_perror_msg("%s", filename);
+
+	src_fd = open_or_warn_stdin(filename);
+	if (src_fd < 0) {
 		return NULL;
 	}
 
-	// figure specific hash algorithims
-	if(ENABLE_MD5SUM && hash_algo==HASH_MD5) {
+	/* figure specific hash algorithims */
+	if (ENABLE_MD5SUM && hash_algo==HASH_MD5) {
 		md5_begin(&context.md5);
 		update = (void (*)(const void*, size_t, void*))md5_hash;
 		final = (void (*)(void*, void*))md5_end;
 		hash_len = 16;
-	} else if(ENABLE_SHA1SUM && hash_algo==HASH_SHA1) {
+	} else if (ENABLE_SHA1SUM && hash_algo==HASH_SHA1) {
 		sha1_begin(&context.sha1);
 		update = (void (*)(const void*, size_t, void*))sha1_hash;
 		final = (void (*)(void*, void*))sha1_end;
 		hash_len = 20;
 	} else {
-		bb_error_msg_and_die("algotithm not supported");
+		bb_error_msg_and_die("algorithm not supported");
 	}
-	
 
-	while(0 < (count = read(src_fd, in_buf, sizeof in_buf))) {
+	while (0 < (count = safe_read(src_fd, in_buf, 4096))) {
 		update(in_buf, count, &context);
 	}
 
-	if(count == 0) {
+	if (count == 0) {
 		final(in_buf, &context);
 		hash_value = hash_bin_to_hex(in_buf, hash_len);
 	}
-	
+
 	RELEASE_CONFIG_BUFFER(in_buf);
-	
-	if(src_fd != STDIN_FILENO) {
+
+	if (src_fd != STDIN_FILENO) {
 		close(src_fd);
 	}
-	
+
 	return hash_value;
 }
 
-/* This could become a common function for md5 as well, by using md5_stream */
-static int hash_files(int argc, char **argv, hash_algo_t hash_algo)
+int md5_sha1_sum_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
+int md5_sha1_sum_main(int argc ATTRIBUTE_UNUSED, char **argv)
 {
 	int return_value = EXIT_SUCCESS;
 	uint8_t *hash_value;
-	unsigned int flags;
+	unsigned flags;
+	hash_algo_t hash_algo = ENABLE_MD5SUM
+		? (ENABLE_SHA1SUM ? (applet_name[0] == 'm' ? HASH_MD5 : HASH_SHA1) : HASH_MD5)
+		: HASH_SHA1;
 
 	if (ENABLE_FEATURE_MD5_SHA1_SUM_CHECK)
-		flags = bb_getopt_ulflags(argc, argv, "scw");
+		flags = getopt32(argv, "scw");
+	else optind = 1;
+	argv += optind;
+	//argc -= optind;
+	if (!*argv)
+		*--argv = (char*)"-";
 
 	if (ENABLE_FEATURE_MD5_SHA1_SUM_CHECK && !(flags & FLAG_CHECK)) {
 		if (flags & FLAG_SILENT) {
 			bb_error_msg_and_die
-				("the -s option is meaningful only when verifying checksums");
+				("-%c is meaningful only when verifying checksums", 's');
 		} else if (flags & FLAG_WARN) {
 			bb_error_msg_and_die
-				("the -w option is meaningful only when verifying checksums");
+				("-%c is meaningful only when verifying checksums", 'w');
 		}
 	}
 
-	if (argc == optind) {
-		argv[argc++] = "-";
-	}
-	
-	if (ENABLE_FEATURE_MD5_SHA1_SUM_CHECK && flags & FLAG_CHECK) {
+	if (ENABLE_FEATURE_MD5_SHA1_SUM_CHECK && (flags & FLAG_CHECK)) {
 		FILE *pre_computed_stream;
 		int count_total = 0;
 		int count_failed = 0;
-		char *file_ptr = argv[optind];
 		char *line;
 
-		if (optind + 1 != argc) {
+		if (argv[1]) {
 			bb_error_msg_and_die
 				("only one argument may be specified when using -c");
 		}
 
-		if (strcmp(file_ptr, "-") == 0) {
-			pre_computed_stream = stdin;
-		} else {
-			pre_computed_stream = bb_xfopen(file_ptr, "r");
-		}
+		pre_computed_stream = xfopen_stdin(argv[0]);
 
-		while ((line = bb_get_chomped_line_from_file(pre_computed_stream)) != NULL) {
+		while ((line = xmalloc_getline(pre_computed_stream)) != NULL) {
 			char *filename_ptr;
 
 			count_total++;
 			filename_ptr = strstr(line, "  ");
+			/* handle format for binary checksums */
+			if (filename_ptr == NULL) {
+				filename_ptr = strstr(line, " *");
+			}
 			if (filename_ptr == NULL) {
 				if (flags & FLAG_WARN) {
-					bb_error_msg("Invalid format");
+					bb_error_msg("invalid format");
 				}
 				count_failed++;
 				return_value = EXIT_FAILURE;
@@ -167,35 +155,21 @@ static int hash_files(int argc, char **argv, hash_algo_t hash_algo)
 			bb_error_msg("WARNING: %d of %d computed checksums did NOT match",
 						 count_failed, count_total);
 		}
-		if (bb_fclose_nonstdin(pre_computed_stream) == EOF) {
-			bb_perror_msg_and_die("Couldnt close file %s", file_ptr);
+		/*
+		if (fclose_if_not_stdin(pre_computed_stream) == EOF) {
+			bb_perror_msg_and_die("cannot close file %s", file_ptr);
 		}
+		*/
 	} else {
-		while (optind < argc) {
-			char *file_ptr = argv[optind++];
-
-			hash_value = hash_file(file_ptr, hash_algo);
+		do {
+			hash_value = hash_file(*argv, hash_algo);
 			if (hash_value == NULL) {
 				return_value = EXIT_FAILURE;
 			} else {
-				printf("%s  %s\n", hash_value, file_ptr);
+				printf("%s  %s\n", hash_value, *argv);
 				free(hash_value);
 			}
-		}
+		} while (*++argv);
 	}
-	return (return_value);
+	return return_value;
 }
-
-#ifdef CONFIG_MD5SUM
-int md5sum_main(int argc, char **argv)
-{
-	return(hash_files(argc, argv, HASH_MD5));
-}
-#endif
-
-#ifdef CONFIG_SHA1SUM
-int sha1sum_main(int argc, char **argv)
-{
-	return(hash_files(argc, argv, HASH_SHA1));
-}
-#endif
