@@ -14,8 +14,9 @@
 /* This is a NOFORK applet. Be very careful! */
 
 int sync_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int sync_main(int argc, char **argv)
+int sync_main(int argc, char **argv ATTRIBUTE_UNUSED)
 {
+	/* coreutils-6.9 compat */
 	bb_warn_ignoring_args(argc - 1);
 
 	sync();
