@@ -17,13 +17,13 @@
 #define SIZE			8
 
 int strings_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int strings_main(int argc, char **argv)
+int strings_main(int argc ATTRIBUTE_UNUSED, char **argv)
 {
 	int n, c, status = EXIT_SUCCESS;
 	unsigned opt;
 	unsigned count;
 	off_t offset;
-	FILE *file = stdin;
+	FILE *file;
 	char *string;
 	const char *fmt = "%s: ";
 	const char *n_arg = "4";
@@ -40,16 +40,14 @@ int strings_main(int argc, char **argv)
 	if (!*argv) {
 		fmt = "{%s}: ";
 		*--argv = (char *)bb_msg_standard_input;
-		goto PIPE;
 	}
 
 	do {
-		file = fopen_or_warn(*argv, "r");
+		file = fopen_or_warn_stdin(*argv);
 		if (!file) {
 			status = EXIT_FAILURE;
 			continue;
 		}
- PIPE:
 		offset = 0;
 		count = 0;
 		do {
