@@ -1,3 +1,4 @@
+/* vi: set sw=4 ts=4: */
 /*
  * libnetlink.c	RTnetlink service routines.
  *
@@ -11,61 +12,41 @@
  */
 
 #include <sys/socket.h>
-
-#include <errno.h>
-#include <stdio.h>
-#include <string.h>
-#include <time.h>
-#include <unistd.h>
-
 #include <sys/uio.h>
 
-#include "libnetlink.h"
 #include "libbb.h"
+#include "libnetlink.h"
 
 void rtnl_close(struct rtnl_handle *rth)
 {
 	close(rth->fd);
 }
 
-int rtnl_open(struct rtnl_handle *rth, unsigned subscriptions)
+int xrtnl_open(struct rtnl_handle *rth/*, unsigned subscriptions*/)
 {
 	socklen_t addr_len;
 
 	memset(rth, 0, sizeof(rth));
 
-	rth->fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
-	if (rth->fd < 0) {
-		bb_perror_msg("Cannot open netlink socket");
-		return -1;
-	}
+	rth->fd = xsocket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
 
 	memset(&rth->local, 0, sizeof(rth->local));
 	rth->local.nl_family = AF_NETLINK;
-	rth->local.nl_groups = subscriptions;
+	/*rth->local.nl_groups = subscriptions;*/
 
-	if (bind(rth->fd, (struct sockaddr*)&rth->local, sizeof(rth->local)) < 0) {
-		bb_perror_msg("Cannot bind netlink socket");
-		return -1;
-	}
+	xbind(rth->fd, (struct sockaddr*)&rth->local, sizeof(rth->local));
 	addr_len = sizeof(rth->local);
-	if (getsockname(rth->fd, (struct sockaddr*)&rth->local, &addr_len) < 0) {
-		bb_perror_msg("Cannot getsockname");
-		return -1;
-	}
-	if (addr_len != sizeof(rth->local)) {
-		bb_error_msg("Wrong address length %d", addr_len);
-		return -1;
-	}
-	if (rth->local.nl_family != AF_NETLINK) {
-		bb_error_msg("Wrong address family %d", rth->local.nl_family);
-		return -1;
-	}
+	if (getsockname(rth->fd, (struct sockaddr*)&rth->local, &addr_len) < 0)
+		bb_perror_msg_and_die("cannot getsockname");
+	if (addr_len != sizeof(rth->local))
+		bb_error_msg_and_die("wrong address length %d", addr_len);
+	if (rth->local.nl_family != AF_NETLINK)
+		bb_error_msg_and_die("wrong address family %d", rth->local.nl_family);
 	rth->seq = time(NULL);
 	return 0;
 }
 
-int rtnl_wilddump_request(struct rtnl_handle *rth, int family, int type)
+int xrtnl_wilddump_request(struct rtnl_handle *rth, int family, int type)
 {
 	struct {
 		struct nlmsghdr nlh;
@@ -83,7 +64,8 @@ int rtnl_wilddump_request(struct rtnl_handle *rth, int family, int type)
 	req.nlh.nlmsg_seq = rth->dump = ++rth->seq;
 	req.g.rtgen_family = family;
 
-	return sendto(rth->fd, (void*)&req, sizeof(req), 0, (struct sockaddr*)&nladdr, sizeof(nladdr));
+	return xsendto(rth->fd, (void*)&req, sizeof(req),
+				 (struct sockaddr*)&nladdr, sizeof(nladdr));
 }
 
 int rtnl_send(struct rtnl_handle *rth, char *buf, int len)
@@ -93,7 +75,7 @@ int rtnl_send(struct rtnl_handle *rth, char *buf, int len)
 	memset(&nladdr, 0, sizeof(nladdr));
 	nladdr.nl_family = AF_NETLINK;
 
-	return sendto(rth->fd, buf, len, 0, (struct sockaddr*)&nladdr, sizeof(nladdr));
+	return xsendto(rth->fd, buf, len, (struct sockaddr*)&nladdr, sizeof(nladdr));
 }
 
 int rtnl_dump_request(struct rtnl_handle *rth, int type, void *req, int len)
@@ -120,15 +102,16 @@ int rtnl_dump_request(struct rtnl_handle *rth, int type, void *req, int len)
 	return sendmsg(rth->fd, &msg, 0);
 }
 
-int rtnl_dump_filter(struct rtnl_handle *rth,
-		     int (*filter)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
-		     void *arg1,
-		     int (*junk)(struct sockaddr_nl *,struct nlmsghdr *n, void *),
-		     void *arg2)
+static int rtnl_dump_filter(struct rtnl_handle *rth,
+		int (*filter)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
+		void *arg1/*,
+		int (*junk)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
+		void *arg2*/)
 {
-	char	buf[8192];
+	int retval = -1;
+	char *buf = xmalloc(8*1024); /* avoid big stack buffer */
 	struct sockaddr_nl nladdr;
-	struct iovec iov = { buf, sizeof(buf) };
+	struct iovec iov = { buf, 8*1024 };
 
 	while (1) {
 		int status;
@@ -151,7 +134,7 @@ int rtnl_dump_filter(struct rtnl_handle *rth,
 		}
 		if (status == 0) {
 			bb_error_msg("EOF on netlink");
-			return -1;
+			goto ret;
 		}
 		if (msg.msg_namelen != sizeof(nladdr)) {
 			bb_error_msg_and_die("sender address length == %d", msg.msg_namelen);
@@ -164,17 +147,18 @@ int rtnl_dump_filter(struct rtnl_handle *rth,
 			if (nladdr.nl_pid != 0 ||
 			    h->nlmsg_pid != rth->local.nl_pid ||
 			    h->nlmsg_seq != rth->dump) {
-				if (junk) {
-					err = junk(&nladdr, h, arg2);
-					if (err < 0) {
-						return err;
-					}
-				}
+//				if (junk) {
+//					err = junk(&nladdr, h, arg2);
+//					if (err < 0) {
+//						retval = err;
+//						goto ret;
+//					}
+//				}
 				goto skip_it;
 			}
 
 			if (h->nlmsg_type == NLMSG_DONE) {
-				return 0;
+				goto ret_0;
 			}
 			if (h->nlmsg_type == NLMSG_ERROR) {
 				struct nlmsgerr *l_err = (struct nlmsgerr*)NLMSG_DATA(h);
@@ -184,37 +168,60 @@ int rtnl_dump_filter(struct rtnl_handle *rth,
 					errno = -l_err->error;
 					bb_perror_msg("RTNETLINK answers");
 				}
-				return -1;
+				goto ret;
 			}
 			err = filter(&nladdr, h, arg1);
 			if (err < 0) {
-				return err;
+				retval = err;
+				goto ret;
 			}
 
-skip_it:
+ skip_it:
 			h = NLMSG_NEXT(h, status);
 		}
 		if (msg.msg_flags & MSG_TRUNC) {
-			bb_error_msg("Message truncated");
+			bb_error_msg("message truncated");
 			continue;
 		}
 		if (status) {
-			bb_error_msg_and_die("!!!Remnant of size %d", status);
+			bb_error_msg_and_die("remnant of size %d!", status);
 		}
-	}
+	} /* while (1) */
+ ret_0:
+	retval++; /* = 0 */
+ ret:
+	free(buf);
+	return retval;
 }
 
-int rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, pid_t peer,
-	      unsigned groups, struct nlmsghdr *answer,
-	      int (*junk)(struct sockaddr_nl *,struct nlmsghdr *n, void *),
+int xrtnl_dump_filter(struct rtnl_handle *rth,
+		int (*filter)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
+		void *arg1)
+{
+	int ret = rtnl_dump_filter(rth, filter, arg1/*, NULL, NULL*/);
+	if (ret < 0)
+		bb_error_msg_and_die("dump terminated");
+	return ret;
+}
+
+int rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n,
+	      pid_t peer, unsigned groups,
+	      struct nlmsghdr *answer,
+	      int (*junk)(struct sockaddr_nl *, struct nlmsghdr *n, void *),
 	      void *jarg)
 {
+/* bbox doesn't use parameters no. 3, 4, 6, 7, they are stubbed out */
+#define peer   0
+#define groups 0
+#define junk   NULL
+#define jarg   NULL
+	int retval = -1;
 	int status;
 	unsigned seq;
 	struct nlmsghdr *h;
 	struct sockaddr_nl nladdr;
 	struct iovec iov = { (void*)n, n->nlmsg_len };
-	char   buf[8192];
+	char   *buf = xmalloc(8*1024); /* avoid big stack buffer */
 	struct msghdr msg = {
 		(void*)&nladdr, sizeof(nladdr),
 		&iov,	1,
@@ -224,8 +231,8 @@ int rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, pid_t peer,
 
 	memset(&nladdr, 0, sizeof(nladdr));
 	nladdr.nl_family = AF_NETLINK;
-	nladdr.nl_pid = peer;
-	nladdr.nl_groups = groups;
+//	nladdr.nl_pid = peer;
+//	nladdr.nl_groups = groups;
 
 	n->nlmsg_seq = seq = ++rtnl->seq;
 	if (answer == NULL) {
@@ -234,14 +241,14 @@ int rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, pid_t peer,
 	status = sendmsg(rtnl->fd, &msg, 0);
 
 	if (status < 0) {
-		bb_perror_msg("Cannot talk to rtnetlink");
-		return -1;
+		bb_perror_msg("cannot talk to rtnetlink");
+		goto ret;
 	}
 
 	iov.iov_base = buf;
 
 	while (1) {
-		iov.iov_len = sizeof(buf);
+		iov.iov_len = 8*1024;
 		status = recvmsg(rtnl->fd, &msg, 0);
 
 		if (status < 0) {
@@ -253,33 +260,34 @@ int rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, pid_t peer,
 		}
 		if (status == 0) {
 			bb_error_msg("EOF on netlink");
-			return -1;
+			goto ret;
 		}
 		if (msg.msg_namelen != sizeof(nladdr)) {
 			bb_error_msg_and_die("sender address length == %d", msg.msg_namelen);
 		}
 		for (h = (struct nlmsghdr*)buf; status >= sizeof(*h); ) {
-			int l_err;
+//			int l_err;
 			int len = h->nlmsg_len;
 			int l = len - sizeof(*h);
 
-			if (l<0 || len>status) {
+			if (l < 0 || len > status) {
 				if (msg.msg_flags & MSG_TRUNC) {
-					bb_error_msg("Truncated message");
-					return -1;
+					bb_error_msg("truncated message");
+					goto ret;
 				}
-				bb_error_msg_and_die("!!!malformed message: len=%d", len);
+				bb_error_msg_and_die("malformed message: len=%d!", len);
 			}
 
 			if (nladdr.nl_pid != peer ||
 			    h->nlmsg_pid != rtnl->local.nl_pid ||
 			    h->nlmsg_seq != seq) {
-				if (junk) {
-					l_err = junk(&nladdr, h, jarg);
-					if (l_err < 0) {
-						return l_err;
-					}
-				}
+//				if (junk) {
+//					l_err = junk(&nladdr, h, jarg);
+//					if (l_err < 0) {
+//						retval = l_err;
+//						goto ret;
+//					}
+//				}
 				continue;
 			}
 
@@ -288,38 +296,43 @@ int rtnl_talk(struct rtnl_handle *rtnl, struct nlmsghdr *n, pid_t peer,
 				if (l < sizeof(struct nlmsgerr)) {
 					bb_error_msg("ERROR truncated");
 				} else {
-					errno = -err->error;
+					errno = - err->error;
 					if (errno == 0) {
 						if (answer) {
 							memcpy(answer, h, h->nlmsg_len);
 						}
-						return 0;
+						goto ret_0;
 					}
 					bb_perror_msg("RTNETLINK answers");
 				}
-				return -1;
+				goto ret;
 			}
 			if (answer) {
 				memcpy(answer, h, h->nlmsg_len);
-				return 0;
+				goto ret_0;
 			}
 
-			bb_error_msg("Unexpected reply!!!");
+			bb_error_msg("unexpected reply!");
 
 			status -= NLMSG_ALIGN(len);
 			h = (struct nlmsghdr*)((char*)h + NLMSG_ALIGN(len));
 		}
 		if (msg.msg_flags & MSG_TRUNC) {
-			bb_error_msg("Message truncated");
+			bb_error_msg("message truncated");
 			continue;
 		}
 		if (status) {
-			bb_error_msg_and_die("!!!Remnant of size %d", status);
+			bb_error_msg_and_die("remnant of size %d!", status);
 		}
-	}
+	} /* while (1) */
+ ret_0:
+	retval++; /* = 0 */
+ ret:
+	free(buf);
+	return retval;
 }
 
-int addattr32(struct nlmsghdr *n, int maxlen, int type, __u32 data)
+int addattr32(struct nlmsghdr *n, int maxlen, int type, uint32_t data)
 {
 	int len = RTA_LENGTH(4);
 	struct rtattr *rta;
@@ -348,7 +361,7 @@ int addattr_l(struct nlmsghdr *n, int maxlen, int type, void *data, int alen)
 	return 0;
 }
 
-int rta_addattr32(struct rtattr *rta, int maxlen, int type, __u32 data)
+int rta_addattr32(struct rtattr *rta, int maxlen, int type, uint32_t data)
 {
 	int len = RTA_LENGTH(4);
 	struct rtattr *subrta;
@@ -390,7 +403,7 @@ int parse_rtattr(struct rtattr *tb[], int max, struct rtattr *rta, int len)
 		rta = RTA_NEXT(rta,len);
 	}
 	if (len) {
-		bb_error_msg("!!!Deficit %d, rta_len=%d", len, rta->rta_len);
+		bb_error_msg("deficit %d, rta_len=%d!", len, rta->rta_len);
 	}
 	return 0;
 }
