@@ -129,10 +129,11 @@ static uint8_t* alloc_dhcp_option(int code, const char *str, int extra)
 
 
 int udhcpc_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int udhcpc_main(int argc, char **argv)
+int udhcpc_main(int argc ATTRIBUTE_UNUSED, char **argv)
 {
 	uint8_t *temp, *message;
-	char *str_c, *str_V, *str_h, *str_F, *str_r, *str_T, *str_A, *str_t;
+	char *str_c, *str_V, *str_h, *str_F, *str_r;
+	USE_FEATURE_UDHCP_PORT(char *str_P;)
 	llist_t *list_O = NULL;
 #if ENABLE_FEATURE_UDHCPC_ARPING
 	char *str_W;
@@ -181,6 +182,7 @@ int udhcpc_main(int argc, char **argv)
 		OPT_a = 1 << 20,
 		OPT_W = 1 << 21,
 #endif
+		OPT_P = 1 << 22,
 	};
 #if ENABLE_GETOPT_LONG
 	static const char udhcpc_longopts[] ALIGN1 =
@@ -207,24 +209,35 @@ int udhcpc_main(int argc, char **argv)
 		"arping\0"         No_argument       "a"
 #endif
 		"request-option\0" Required_argument "O"
+#if ENABLE_FEATURE_UDHCP_PORT
+		"client-port\0"	   Required_argument "P"
+#endif
 		;
 #endif
 	/* Default options. */
+#if ENABLE_FEATURE_UDHCP_PORT
+	SERVER_PORT = 67;
+	CLIENT_PORT = 68;
+#endif
 	client_config.interface = "eth0";
 	client_config.script = DEFAULT_SCRIPT;
 
 	/* Parse command line */
-	opt_complementary = "c--C:C--c:O::"; // Cc: mutually exclusive; O: list
+	/* Cc: mutually exclusive; O: list; -T,-t,-A take numeric param */
+	opt_complementary = "c--C:C--c:O::T+:t+:A+";
 #if ENABLE_GETOPT_LONG
 	applet_long_options = udhcpc_longopts;
 #endif
 	opt = getopt32(argv, "c:CV:fbH:h:F:i:np:qRr:s:T:t:vSA:"
 		USE_FEATURE_UDHCPC_ARPING("aW:")
+		USE_FEATURE_UDHCP_PORT("P:")
 		"O:"
 		, &str_c, &str_V, &str_h, &str_h, &str_F
 		, &client_config.interface, &client_config.pidfile, &str_r
-		, &client_config.script, &str_T, &str_t, &str_A
+		, &client_config.script
+		, &discover_timeout, &discover_retries, &tryagain_timeout
 		USE_FEATURE_UDHCPC_ARPING(, &str_W)
+		USE_FEATURE_UDHCP_PORT(, &str_P)
 		, &list_O
 		);
 
@@ -262,12 +275,9 @@ int udhcpc_main(int argc, char **argv)
 	if (opt & OPT_r)
 		requested_ip = inet_addr(str_r);
 	// if (opt & OPT_s) client_config.script = ...
-	if (opt & OPT_T)
-		discover_timeout = xatoi_u(str_T);
-	if (opt & OPT_t)
-		discover_retries = xatoi_u(str_t);
-	if (opt & OPT_A)
-		tryagain_timeout = xatoi_u(str_A);
+	// if (opt & OPT_T) discover_timeout = xatoi_u(str_T);
+	// if (opt & OPT_t) discover_retries = xatoi_u(str_t);
+	// if (opt & OPT_A) tryagain_timeout = xatoi_u(str_A);
 	if (opt & OPT_v) {
 		puts("version "BB_VER);
 		return 0;
@@ -276,6 +286,12 @@ int udhcpc_main(int argc, char **argv)
 		openlog(applet_name, LOG_PID, LOG_LOCAL0);
 		logmode |= LOGMODE_SYSLOG;
 	}
+#if ENABLE_FEATURE_UDHCP_PORT
+	if (opt & OPT_P) {
+		CLIENT_PORT = xatou16(str_P);
+		SERVER_PORT = CLIENT_PORT - 1;
+	}
+#endif
 	while (list_O) {
 		int n = index_in_strings(dhcp_option_strings, list_O->data);
 		if (n < 0)
@@ -452,9 +468,10 @@ int udhcpc_main(int argc, char **argv)
 
 			if (listen_mode == LISTEN_KERNEL)
 				len = udhcp_recv_packet(&packet, sockfd);
-			else len = get_raw_packet(&packet, sockfd);
+			else
+				len = get_raw_packet(&packet, sockfd);
 
-			if (len == -1 && errno != EINTR) {
+			if (len == -1) { /* error is severe, reopen socket */
 				DEBUG("error on read, %s, reopening socket", strerror(errno));
 				change_listen_mode(listen_mode); /* just close and reopen */
 			}
@@ -523,7 +540,7 @@ int udhcpc_main(int argc, char **argv)
 						) {
 							bb_info_msg("offered address is in use "
 								"(got ARP reply), declining");
-							send_decline(xid, server_addr);
+							send_decline(xid, server_addr, packet.yiaddr);
 
 							if (state != REQUESTING)
 								udhcp_run_script(NULL, "deconfig");
